from pywyse.client import Client 
from pywyse.exceptions import PywyseException
import os, tempfile


class Invoicing(Client): 
    def __init__(self):
        super(Invoicing, self).__init__()
    
    def get_invoices(self, endpoint='/finance/invoices', **kwargs): 
        return self.cw_get(endpoint, **kwargs).json()

    def get_invoice(self, invoice, **kwargs): 
        kwargs['conditions'] = f"invoiceNumber = '{invoice}'"
        kwargs['pageSize'] = '1'

        return self.get_invoices(**kwargs)
    
    def get_invoice_pdf(self, invoice, **kwargs):
        inv = self.get_invoice(invoice) 
        endpoint = '/finance/invoices/' + str(inv[0]['id']) + '/pdf'

        kwargs['headers'] = {'Accept': 'application/pdf'}
        resp = self.cw_get(endpoint, **kwargs)
        
        tmp = tempfile.mkstemp(suffix='.pdf', prefix=str(invoice))
        
        with open(tmp[1], 'wb') as s_inv:
            s_inv.write(resp.content)
    
        return {"invoice": invoice, "path": tmp[1]}


from .models import ProcurementModel

from pywyse.client import Client
from pywyse.service import Ticket
from pywyse.exceptions import PywyseException

class Procurement(Client): 

    def __init__(self): 
        super(Procurement, self).__init__()


    def get_purchaseorders(self, endpoint = '/procurement/purchaseorders', **kwargs) -> list:
        """
        Returns a list of dictionaries.
        """
        return self.cw_get(endpoint=endpoint, **kwargs).json()

    def get_purchaseorder(self, purchase_order=None, **kwargs) -> dict:
        """
        Should only return a single purchase order. No match will raise an exception.
        """

        kwargs['conditions'] = "poNumber='C9PO%s'" % purchase_order
        kwargs['pageSize'] = 1
        kwargs['fields'] = 'id,vendorCompany/name'

        resp = self.get_purchaseorders(**kwargs)

        try: 
            return resp[0]
        except IndexError:
            raise PywyseException("Invalid purchase order")
        

    def get_purchaseorder_lines(self, purchase_order, **kwargs) -> list: 
        po_id = self.get_purchaseorder(purchase_order)
        endpoint = f"/procurement/purchaseorders/{po_id['id']}/lineitems"
        kwargs['fields'] = "id,quantity,product/identifier,description,internalNotes,receivedStatus"
 
        resp = self.cw_get(endpoint=endpoint, **kwargs).json()
      
        for i in range(len(resp)):
            resp[i]['product'] = resp[i]["product"]["identifier"]
            resp[i]['po_id'] = po_id['id']
            resp[i]['vendorName'] = po_id['vendorCompany']['name']

        return resp
    
    def get_purchaseorder_line(self, po_id, line_id, **kwargs): 
        
        endpoint = f"/procurement/purchaseorders/{po_id}/lineitems/{line_id}"
        kwargs['fields'] = "id,quantity"
        kwargs['pageSize'] = "1"
 
        resp = self.cw_get(endpoint=endpoint, **kwargs).json()
      
        return resp


    def get_product_meta(self, ticket, **kwargs):
        """
        Product response: 
            [{'project': {'id': 5981, 'name': 'AV'}, 'company': {'name': 'Rosenberg, Robert Residence'}}]
        Subcontractor:
            [{'company': {'name': 'Hilldun Corp'}, 'summary': 'SUB:  36 East 31st Cabling:  PO Request: Lightspeed'}]
        """

        purch_tick = Ticket().get_ticket(ticket)

        kwargs['conditions'] = f"company/id = {purch_tick['company']} and internalNotes contains '{ticket}'"
        kwargs['fields'] = 'project/name,project/id,company/name'
        kwargs['pageSize'] = '1'
       
        endpoint = '/procurement/products'
        resp = self.cw_get(endpoint, **kwargs).json()

        resp[0]['summary'] = purch_tick.get('summary', '')

        return resp 


    def receive_product(self, purchase_id, line_id, receive_quantity, **kwargs): 
        """
        Add a decorator to send to slack. 
        Patch: /procurement/purchaseorders/{parentId}/lineitems/{id}
        {'id': '17443', 'internalNotes': None, 'product': 'QS-WLB', 'quantity': 5, 'quantityReceived': 0}

      
        When receiving partial quantity on a fully received item, a new item with quantity: 0 will be created.
        """
        # Test if it's fully or partially received
        line_qty = self.get_purchaseorder_line(purchase_id, line_id)

        if line_qty['quantity'] == receive_quantity:
            status =  'FullyReceived'
        else: 
            status = 'PartiallyReceiveCloneRest'

        kwargs['body'] = [
            {
                'op': 'replace',
                'path': 'receivedQuantity',
                'value': str(receive_quantity)
                }, 
            {
                'op': 'replace',
                'path': 'receivedStatus',
                'value': status
                } 
            ]

        endpoint = f'/procurement/purchaseorders/{str(purchase_id)}/lineitems/{str(line_id)}'

        try: 
            resp = self.cw_patch(endpoint, **kwargs)
            
        except Exception as e: 
            print("Error closing poid: %s" % purchase_id)
            print(e)
            resp = False

        return resp 

    def purchase_order(self, endpoint='/procurement/purchaseorders', **kwargs): 
        resp = self.cw_post(endpoint, **kwargs)
        return resp

# class PurchaseOrder(PO, Client): 
#     def __init__(self, model=''): 



import os 
import re 

from pywyse.exceptions import PywyseException

def filter_ticket(resp: list) -> list:
    """
    Take a list of line items and return unique internal notes as a list.
    [{'id': 17347, 'description': 'Delivery', 'internalNotes': '398828', 'product': 'LSD - 001', 'quantity': 1.0, 'po_id': 7264}]
    """ 
    
    r = [i.get('internalNotes', 'empty') for i in resp]
    resp = list(set([item for sublist in [re.findall(r'^(\d{6})', c) for c in r] for item in sublist]))
    
    if len(resp) == 0:
        raise PywyseException("No response")
        
    return resp 



from pywyse.exceptions import PywyseException
from pywyse.client import Client

class Project(Client): 

    def __init__(self): 
        super(Project, self).__init__()
        self.endpoint = '/project/projects/' 

    def get_projects(self, endpoint='',  **kwargs): 
        if not endpoint: 
            endpoint = self.endpoint
        return self.cw_get(endpoint, **kwargs).json()

    def get_project(self, id, **kwargs):
        endpoint = self.endpoint + str(id)
        return self.get_projects(endpoint, **kwargs)

    def get_open_projects(self, conditions='closedFlag=false', pageSize = 1000, **kwargs): 
        return self.get_projects(pageSize=pageSize, conditions=conditions, **kwargs)
    
    def get_products(self, project_id, **kwargs):
        project_id = 'project/id=%s' % str(project_id)
        if kwargs.get('conditions'): 
            kwargs['conditions'] = project_id + ' and ' + kwargs['conditions']         
        else: 
            kwargs['conditions'] = project_id
            
        endpoint = '/procurement/products/'

        resp = self.cw_get(endpoint, **kwargs).json()
        try: 
            for i in range(len(resp)):
                resp[i]['picked'] = self.get_product_pickedstatus(resp[i]['id'])
            return resp 
        except Exception: 
            return resp 
        


    def get_product_pickedstatus(self, product_id, **kwargs): 
        endpoint = '/procurement/products/' + str(product_id) + '/pickingShippingDetails/'
        resp = self.cw_get(endpoint, fields='pickedQuantity').json()
        try: 
            return resp[0].get('pickedQuantity', '0')
        except: 
            return resp 



from pywyse import project
from pywyse.service import Ticket
from pywyse.service.models import PurchaseTicket
import pytest 

def test_post(): 
    tickets = Ticket().get_tickets(pageSize=1000, page=20)
    assert isinstance(tickets, list)
    assert len(tickets) == 1000

    ticket = Ticket().get_ticket(tickets[999]['id'])
    assert isinstance(ticket, dict)

    # Defaults to C9Smart company
    simple_ticket = PurchaseTicket(summary="HelloWorld!").to_dict()

    # Project ticket with no notes 
    project_ticket = PurchaseTicket(
        summary='ProjectInfo',
        company={'id':'2'}, 
        project_id='5866'
        ).to_dict()

    # Project ticket with notes 
    project_ticket_notes = PurchaseTicket(
        summary='ProjectInfo',
        company={'id':'2'}, 
        project_id='5866',
        notes='Project ticket with notes'
        ).to_dict()

    
    # resp = Ticket().create_ticket(Ticket=simple_ticket)
    # resp1 = Ticket().create_ticket(Ticket=project_ticket)
    # resp2 = Ticket().create_ticket(Ticket=project_ticket_notes)
    
    # ticket_notes = Ticket().get_ticket_association(resp2)
    # assert isinstance(ticket_notes, list) 




import pytest 
from pywyse import Client 

def test_package_import():
    import pywyse 
    pywyse.client 
    pywyse.service
    pywyse.procurement
    
    with pytest.raises(AttributeError):
        pywyse.someerror

def test_function_import(): 
    from pywyse.procurement import Procurement
    from pywyse import procurement
    procurement.Procurement
    import pywyse 
    pywyse.procurement.Procurement

    # from pywyse import service 
    # service.get_ticket
    from pywyse import company
    company.Company
    from pywyse.company import Company

def test_build_url(): 
    resp = Client.build_url('/hello/world', 'myname/is')
    assert isinstance(resp, str)
    assert resp == '/hello/world/myname/is'


import pytest 
from pywyse.utils import filter_ticket
from pywyse.exceptions import PywyseException

x = [
    {'id': 17347, 'description': 'Delivery', 'internalNotes': '398828', 'product': 'LSD - 001', 'quantity': 1.0, 'po_id': 7264},
    {'id': 17347, 'description': 'Delivery', 'internalNotes': 'This isome kjelr398828', 'product': 'LSD - 001', 'quantity': 1.0, 'po_id': 7264},
    {'id': 17347, 'description': 'Delivery', 'internalNotes': '398828Thisaslkjasd', 'product': 'LSD - 001', 'quantity': 1.0, 'po_id': 7264},
    {'id': 17347, 'description': 'Delivery', 'internalNotes': 'PartNo321\n398828 More text\n\n', 'product': 'LSD - 001', 'quantity': 1.0, 'po_id': 7264},
    {'id': 17347, 'description': 'Delivery', 'internalNotes': '3123121g2\b\bPartNo321\n398828 More text\n\n', 'product': 'LSD - 001', 'quantity': 1.0, 'po_id': 7264},
    {'id': 17347, 'description': 'Delivery', 'internalNotes': '', 'product': 'LSD - 001', 'quantity': 1.0, 'po_id': 7264},
    ]

y = [{'id': 17347, 'description': 'Delivery', 'internalNotes': '', 'product': 'LSD - 001', 'quantity': 1.0, 'po_id': 7264},]

p = [
    {'id': 16813, 'description': 'Cisco Meraki MX84 Cloud Managed - Firewall', 'internalNotes': '394963', 'product': 'MX84-HW', 'quantity': 1.0, 'po_id': 7026}, 
    {'id': 16815, 'description': 'Cisco Meraki MS120-48LP Cloud Managed - 48P Network Switch', 'internalNotes': '394963', 'product': 'MS120-48LP-HW', 'quantity': 1.0, 'po_id': 7026},
    {'id': 16816, 'description': 'Meraki Enterprise MS120-48LP - ', 'internalNotes': '394963', 'product': 'LIC-MS120-48LP-1YR', 'quantity': 1.0, 'po_id': 7026},
    {'id': 16817, 'description': 'Cisco Meraki MS120-24LP Cloud Managed - 24P Network Switch', 'internalNotes': '394963', 'product': 'MS120-24P-HW', 'quantity': 2.0, 'po_id': 7026},
    {'id': 16818, 'description': 'Meraki Enterprise MS120-24LP - Subscription License ( 1 Year )', 'internalNotes': '394963', 'product': 'LIC-MS120-24P-1YR', 'quantity': 2.0, 'po_id': 7026}, 
    {'id': 16819, 'description': 'Cisco Meraki MR42 Cloud Managed Wireless Access Point', 'internalNotes': '394963', 'product': 'MR42-HW', 'quantity': 7.0, 'po_id': 7026}, 
    {'id': 16821, 'description': 'Cisco Meraki Outdoor Acess Point - IP67 Rugged ', 'internalNotes': '394963', 'product': 'MR70-HW', 'quantity': 3.0, 'po_id': 7026}]

def test_filter(): 
    filter_ticket(x)
    filter_ticket(p)
    
    with pytest.raises(PywyseException):
        filter_ticket(y)




from pywyse.sell import Sell
from pywyse.project import Project
import pytest 

def test_sell():

    quotes = Sell().get_quotes(pageSize=10)
    assert isinstance(quotes, list)
    assert len(quotes) == 10

    project = Project().get_open_projects()
    sell_id = Sell().translate(project[10]['opportunity'].get('id', ''))
    assert isinstance(sell_id, list)
    assert len(sell_id) == 1

    sell_quote = Sell().get_quote(id = sell_id[0]['id'])
    assert isinstance(sell_quote, dict)
    



from pywyse.sales import Opportunity
import pytest 

def test_sales(): 

    opps = Opportunity().get_opportunities(pageSize = 32)
    assert isinstance(opps, list)
    assert len(opps) == 32

    opp = Opportunity().get_opportunity(opps[30]['id'])
    assert isinstance(opp, list)
    assert len(opp) == 1

    


import requests
import os 
from pywyse import __conf

sell_conf = __conf

class Sell: 
    def __init__(self, conf=sell_conf): 
        self.conf=conf

    def base_url(self, endpoint=''): 
        url = f"https://sellapi.quosalsell.com"
        url = url + endpoint 
        return url 
    
    def headers(self, **kwargs): 
        headers = {
            'Content-Type': 'application/json',
            'Authorization': self.conf['sell_auth'],
            **kwargs 
        }
        return headers 
    
    def get_sell(self, endpoint='', verbose=True, **kwargs):
        headers = kwargs.pop('headers', {})
        headers = self.headers(**headers) 
        if verbose:
            print(endpoint)
            print(headers)
            print(kwargs)
        return requests.get(self.base_url(endpoint), headers=headers, params=kwargs)

    def translate(self, id): 
        """Translate CW Opportunity id to sell id."""
        condition = 'crmOpportunityID = %s' % str(id) 
        includeFields = 'name,quoteNumber,quoteVersion,id,approvalStatus'
        return self.get_quotes(conditions=condition, pageSize=1, includeFields=includeFields)

    def get_quotes(self, **kwargs):
        return self.get_sell('/api/quotes', **kwargs).json()
        

    def get_quote(self, id, **kwargs): 
        endpoint = '/api/quotes/' + str(id)
        return self.get_sell(endpoint, **kwargs).json()



from pywyse.client import Client 

class CallBack(Client): 

    def __init__(self): 
        super(CallBack, self).__init__()
        self.endpoint = '/system/callbacks/' 

    def get_callbacks(self, endpoint='', **kwargs): 
        if not endpoint: 
            endpoint = self.endpoint 
        return self.cw_get(endpoint, **kwargs).json() 

    def get_callback(self, callback, **kwargs): 
        endpoint = self.endpoint + str(callback)
        return self.get_callbacks(endpoint, **kwargs)


    def create(self, callback, **kwargs): 
        """
        From docs: 
            Id: The database record id of the callback, this is automatically assigned.
            Description: This is used to label the callback's usage.
            URL: This is the URL ConnectWise Manage will send the GET payload to.
            ObjectId: The ObjectId should be the Id of whatever record you are subscribing to. 
                This should be set to 1 when using a level of Owner. 
            Type: This is the specific type of record such as Company, Ticket, Contact, 
                etc... See the associated table for all values.
            Level: The level is used to determine how granular the callback subscription will be.
            MemberId: This is a read only value that shows who initially created the Callback.
            InactiveFlag: Used to determine if the callback is active and sending requests.

            Success: status_code == 204
        """
        return self.cw_post(self.endpoint, body=callback, **kwargs)

    def void(self, callback_id, **kwargs):
        endpoint = self.endpoint + str(callback_id)
        return self.cw_delete(endpoint, **kwargs)
    
class Board(Client): 

    def __init__(self): 
        super(Board, self).__init__()
        self.endpoint = '/service/boards/' 

    def get_boards(self, fields='', **kwargs): 
        if not fields: 
            fields='id,name'
        return self.cw_get(self.endpoint, fields=fields, **kwargs).json()

class Member(Client):

    def __init__(self): 
        super(Member, self).__init__()
        self.endpoint = '/system/members'

    def get_members(self, fields='', **kwargs): 
        if not fields:
            fields='id,identifier,firstName,lastName,employeeIdentifer,inactiveFlag'
        return self.cw_get(self.endpoint, fields=fields, **kwargs).json()




from datetime import datetime 
from typing import ValuesView
from pywyse.client import Client 

class Procurement(Client):
    
    def __init__(self, *args, **kwargs): 
        super(Procurement, self).__init__(*args, **kwargs) 
        self.endpoint = '/finance/accounting'
        self.batch_ids = [] 


    def count_unposted(self, **kwargs): 
        """
        Count the number of items in the Procurement Interface to be transferred. 
        """
        endpoint = self.build_url(self.endpoint, 'unpostedprocurement/count/')
        try: 
            resp = self.cw_get(endpoint, **kwargs).json()
        except Exception as e: 
            raise ValueError("Could not connect")

        return resp

    def get_unposted(self, **kwargs): 
        """
        Get the procurement summary to be transferred. 
        """
        endpoint = self.build_url(self.endpoint, 'unpostedprocurement/')
        try: 
            resp = self.cw_get(endpoint, **kwargs).json()
        except Exception as e: 
            raise ValueError('Could not connect')
        
        return resp 

    def get_batch_ids(self, ids=None):
        """
        Batching the current accounting interface
        """ 
        if ids is None:
            resp_interface = Procurement().get_unposted(fields='unpostedProductId', pageSize=1000)
            ids = [c['unpostedProductId'] for c in resp_interface]
        try: 
            self.batch_ids = [c for c in ids]
        except Exception as e: 
            print(e) 
            raise ValueError('Missing ids')


    def soft_batch(self, **kwargs): 
        if not self.batch_ids: 
            self.get_batch_ids()
        
        soft_batch = {
            'batchIdentifier': 'POS' + datetime.now().strftime('%Y%m%d'),
            'exportProductsFlag': True,
            'includedProductIds': self.batch_ids
        }

        endpoint = self.build_url(self.endpoint, 'export/')
        return self.cw_post(endpoint, body=soft_batch).json()
    

    def batch(self, ids, **kwargs): 

        ids = [c for c in ids]
        batch = {
            # What the batch will be named
            'batchIdentifier':'P' + datetime.now().strftime('%y%m%d%H%M%S'),
            'glInterfaceIdentifier':'P' + datetime.now().strftime('%y%m%d%H%M%S'),
            'exportInvoicesFlag':False,
            'exportExpensesFlag':False,
            'exportProductsFlag':True,
            'processedRecordIds':ids,
            'summarizeExpenses':False,
        }
        endpoint = self.build_url(self.endpoint, 'batches')
        self.cw_post(endpoint, body = batch, **kwargs)



import pytest 
from pywyse.invoicing import Invoicing 
from pathlib import Path

def test_invoices(): 
    invoices = Invoicing().get_invoices(pageSize=1000, page=2)

    assert isinstance(invoices, list)
    assert len(invoices) == 1000

    invoice = Invoicing().get_invoice(invoices[0].get('invoiceNumber'))
    assert len(invoice) == 1
    assert isinstance(invoice, list)

    resp = Invoicing().get_invoice_pdf(invoice[0].get('invoiceNumber'))
    assert Path.exists(Path(resp['path']))
    
    


import os 

import requests 
from . import __conf

conf = __conf

class Client: 
    def __init__(self, conf=conf, *args, **kwargs): 
        self.conf = conf
        
    def base_url(self, endpoint=''): 
        url = f"https://{self.conf['cw_server']}/{self.conf['cw_codebase']}/apis/{self.conf['cw_apiversion']}"
        url = url + endpoint 
        return url 
    
    def headers(self, **kwargs):
        headers = {
            'Content-Type': 'application/json',
            'Authorization': self.conf['auth'],
            'clientId': self.conf['cw_clientid'], 
            **kwargs
        } 
        return headers 

    def cw_get(self, endpoint='', verbose=False, **kwargs): 
        endpoint = self.base_url(endpoint=endpoint)
        headers = kwargs.pop('headers', {})
        headers = self.headers(**headers)
        if verbose:
            print(kwargs)
            print(endpoint)
            print(headers)
            
        return requests.get(endpoint, headers=headers, params=kwargs)
        
    def cw_patch(self, endpoint='', verbose=False, **kwargs): 
        endpoint = self.base_url(endpoint=endpoint)
        headers = kwargs.pop('headers', {})
        headers = self.headers(**headers)
        body = kwargs.pop('body', {})
        if verbose:
            print(endpoint)
            print(body)
            print(headers)
        
        return requests.patch(endpoint, headers=headers, json=body)
        
    def cw_put(self, endpoint='', verbose=False, **kwargs): 
        endpoint = self.base_url(endpoint=endpoint)
        headers = kwargs.pop('headers', {})
        headers = self.headers(**headers)
        if verbose:
            print(kwargs)
            print(endpoint)
            print(headers)
        
        return requests.put(endpoint, headers=headers, json=kwargs)
        
    def cw_post(self, endpoint='', verbose=False, body={}, **kwargs): 
        endpoint = self.base_url(endpoint=endpoint)
        headers = kwargs.pop('headers', {})
        headers = self.headers(**headers)
        if verbose:
            print(kwargs)
            print(endpoint)
            print(headers)
            
        return requests.post(endpoint, headers=headers, json=body)
        
    def cw_delete(self, endpoint='', verbose=False, **kwargs): 
        endpoint = self.base_url(endpoint=endpoint)
        headers = kwargs.pop('headers', {})
        headers = self.headers(**headers)
        if verbose:
            print(kwargs)
            print(endpoint)
            print(headers)
            
        return requests.delete(endpoint, headers=headers, **kwargs)

    @staticmethod 
    def build_url(*args):
        return '/'.join(args) 
        




import pytest 
from pywyse.procurement import Procurement 
from pywyse.exceptions import PywyseException


def test_purchaseorder(): 
    p = Procurement()

    pos = p.get_purchaseorders(pageSize=1000, page=5)
    assert isinstance(pos, list)
    assert len(pos) > 0
    assert len(pos) == 1000

    po=pos[999].get('poNumber')

    with pytest.raises(PywyseException):  
        p.get_purchaseorder(po)

    po = p.get_purchaseorder(po[4:])

    assert isinstance(po, dict)
    assert 'id' in po
    # assert hasattr(po, 'id')


from pywyse.company import Company
import pytest 

def test_company(): 
    companies = Company().get_companies(pageSize=32)
    assert isinstance(companies, list)
    assert len(companies) == 32

    company = Company().get_company(companies[30]['id'])
    assert isinstance(company, dict)
    # assert len(company) == 1

    site = Company().get_sites(company['id'])
    assert company.get('billingSite').get('id') in [c['id'] for c in site]
     
    billing_site = Company().get_site(company['id'], company['billingSite']['id'])
    assert isinstance(billing_site, dict)


from dataclasses import asdict, dataclass, field
from typing import Dict, TypedDict
import json 
from typing import Optional
from pywyse.project import Project 
from typing import NamedTuple


@dataclass 
class PurchaseTicket:
    """Creating purchase tickets"""
    summary: str 
    project_description: str = None 
    project_id: str = None
    notes: str = None
    id: str = None
    recordType: str = 'ServiceTicket'
    board: Dict[str,str] = field(default_factory=lambda: ({'name':'Purchasing'}))
    company: Dict[str,str] = field(default_factory=lambda: ({'id':'2'}))
    status: Dict[str,str] = field(default_factory=lambda: ({'name':'New'}))

    def __post_init__(self): 
        if self.project_id is not None:
            r = Project().get_project(self.project_id, field='name')
            self.project_description = f"Project: [{self.project_id}] - {r.get('name','')}"
          
    def to_dict(self): 
        _data = asdict(self)
        return {k:v for k,v in _data.items() if v is not None and k is not 'project_id'}



import pytest 
from pywyse.accounting import Expense
from pywyse.accounting import Invoice
from pywyse.accounting import Procurement
from pywyse.client import Client

from pprint import pprint 


def test_count(): 
    resp = Procurement().count_unposted(verbose=True)
    assert isinstance(resp, dict)
    
def test_pre_batch(): 
    resp = Procurement().get_unposted(verbose = True, pageSize=1)
    assert isinstance(resp, list)
    assert len(resp) == 1
    resp

    pprint(resp)


def test_soft_batch(): 
    prod = Procurement()
    prod.get_batch_ids(ids=['7325-STOCK10012021-purchase'])
    assert len(prod.batch_ids) == 1
    assert isinstance(prod.batch_ids, list)
    

def test_soft_batch_ids(): 
    prod = Procurement()
    resp = prod.soft_batch()
    
    resp.keys()
    pprint(resp['exportSettings'])
    pprint(resp['transactions'])
    pprint(resp['purchaseTransactions'])
    len(resp['purchaseTransactions'])
    pprint(resp['purchaseTransactions'][0])


from pywyse.client import Client 
from pywyse.exceptions import PywyseException


class Opportunity(Client):
    
    def __init__(self):
        super(Opportunity, self).__init__()

    def get_opportunities(self, **kwargs): 
        return self.cw_get('/sales/opportunities', **kwargs).json()

    def get_opportunity(self, id, **kwargs): 
        kwargs['conditions'] = 'id = %s' % id
        kwargs['pageSize'] = '1'
        return self.get_opportunities(**kwargs)


from pywyse.client import Client 

class Company(Client): 

    def __init__(self): 
        super(Company, self).__init__()
        
    def get_companies(self, endpoint='/company/companies', **kwargs): 
        return self.cw_get(endpoint=endpoint, **kwargs).json()
      
    def get_company(self, id=2, **kwargs): 
        endpoint = '/company/companies/' + str(id)
        return self.cw_get(endpoint, **kwargs).json()

    def get_sites(self, company_id, **kwargs):
        endpoint = '/company/companies/' + str(company_id) + '/sites'
        return self.cw_get(endpoint, **kwargs).json() 

    def get_site(self, company_id, site_id, **kwargs):
        endpoint = '/company/companies/' + str(company_id) + '/sites/' + str(site_id)
        return self.cw_get(endpoint, **kwargs).json()




from base64 import b64encode
from pathlib import Path 
import configparser

BASE_DIR = Path.home()

config = configparser.ConfigParser()

def auth(conf, auth='auth', platform='CW'): 
    if platform == 'CW': 
        user = conf['cw_company'] + "+" + conf['cw_public'] + ":" + conf['cw_private']
    else: 
        user = conf['sell_key'] + "+" + conf['sell_user'] + ":" + conf['sell_pw']
    password = b64encode(user.encode('ascii'))  
    conf[auth] = "Basic %s" % password.decode()
    return conf 
       

try: 
    config.read(BASE_DIR / '.cw_conf')
    __conf = {}
    for k in config["DEFAULT"]: 
        __conf[k] = config['DEFAULT'][k]
    for s in config["SELL"]: 
        __conf[s] = config['SELL'][s]

    __conf = auth(conf=__conf)
    __conf = auth(conf=__conf, auth='sell_auth', platform='SELL')
    
except: 
    print('No .cw_conf file found.\n\nPlease update the config file in your home directory.')

    config['CW'] = {
        'CW_SERVER': '',
        'CW_COMPANY': '',
        'CW_PUBLIC': '',
        'CW_PRIVATE': '',
        'CW_CLIENTID': '',
        'CW_CODEBASE': '',
        'CW_APIVERSION': ''
    }
    with open(BASE_DIR / '.cw_conf', 'w') as configfile:
        config.write(configfile)


# from . import auth
from .client import Client 
from . import basemodel
from . import exceptions
from . import utils
from pywyse.agreement import agreement
from pywyse.company import company
from pywyse.expense import expense
from pywyse.finance import finance
from pywyse.invoicing import invoicing
from pywyse.procurement import procurement
from pywyse.project import project
from pywyse.sales import sales
from pywyse.service import service 
from pywyse.system import system 
from pywyse.time import time 




from pywyse.system import CallBack, Member, Board
import pytest 

def test_callback(): 

    resp = CallBack().get_callbacks(pageSize=100)
    assert isinstance(resp, list)
    assert len(resp) == 100
    assert CallBack().endpoint == '/system/callbacks/'
    


def test_boards(): 

    resp = Board().get_boards(pageSize=5)
    assert isinstance(resp, list)
    assert len(resp) == 5 
    
def test_members(): 

    resp = Member().get_members(pageSize=100, conditions="inactiveFlag=False")
    assert isinstance(resp, list)
    
    
    resp = Member().get_members(pageSize=5)
    assert isinstance(resp, list)
    assert len(resp) == 5 


from pywyse.client import Client 
from .models import PurchaseTicket

class Ticket(Client): 

    def __init__(self):
        super(Ticket, self).__init__()
        self.endpoint = '/service/tickets/' 

    def get_tickets(self, endpoint='', **kwargs): 
        if not endpoint:
            endpoint = self.endpoint
        return self.cw_get(endpoint, **kwargs).json()

    def get_ticket(self, ticket, fields='id,summary,company/id,company/name', **kwargs): 
        endpoint = self.endpoint + str(ticket)
        resp = self.get_tickets(endpoint, fields=fields, **kwargs)
        try:
            resp['companyName']= resp['company']['name']
            resp['company']= resp['company']['id']
        except Exception as e: 
            print(e)
            
        return resp

    def get_ticket_association(self, ticket, **kwargs): 
        """Returns the tickets project. Why can't purchase tickets be converted to project
        tickets.
        """
        endpoint = self.endpoint + str(ticket) + '/notes/'
        c = 'member/id=%s' % self.conf.get('cw_integration', '')
        fields = 'text,id'
        print(endpoint)
        resp = self.cw_get(endpoint, conditions=c, fields=fields,**kwargs)
        return resp.json() 

    def create_ticket(self, Ticket, **kwargs): 
        endpoint = self.endpoint

        notes = Ticket.pop('notes', '')
        project_id = Ticket.pop('project_description', '')
        resp = self.cw_post(endpoint, body=Ticket, **kwargs)
        
        if resp.status_code == 201 and project_id: 
            ticket_id = endpoint + '/' + str(resp.json()['id']) + '/notes/'
            print(ticket_id)
            body = {'text': project_id, 'internalAnalysisFlag': True}
            resp = self.cw_post(ticket_id, body=body).json()
            print(resp)
            if notes:
                body = {'text': notes, 'detailDescriptionFlag': True}
                r = self.cw_post(ticket_id, body=body) 
                return r.json()['ticketId']

            else: 
                return resp['ticketId']
        else: 
            try: 
                return resp.json()['id']
            except Exception as e: 
                print(e) 
                return resp



from pywyse.client import Client 


class Time(Client): 

    def __init__(self): 
        super(Time, self).__init__()

    def get_timesheets(self, endpoint='/time/sheets', **kwargs):
        self.cw_get(endpoint, **kwargs)

    def get_timeentries(self, endpoint='/time/entries', **kwargs):
        self.cw_get(endpoint, **kwargs) 